@@ -14,7 +14,8 @@ from cloudinary import utils
 from cloudinary.api_client.call_api import (
     call_api,
     call_metadata_api,
-    call_json_api
+    call_json_api,
+    _call_v2_api
 )
 from cloudinary.exceptions import (
     BadRequest,
@@ -28,6 +29,16 @@ from cloudinary.exceptions import (
 
 
 def ping(**options):
+    """
+    Tests the reachability of the Cloudinary API.
+
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+
+    See: https://cloudinary.com/documentation/admin_api#ping
+    """
     return call_api("get", ["ping"], {}, **options)
 
 
@@ -38,7 +49,7 @@ def usage(**options):
     requests, number of resources, and add-on usage. Note that numbers are updated periodically.
 
     See: `Get account usage details
-    <https://cloudinary.com/documentation/admin_api#get_account_usage_details>`_
+    <https://cloudinary.com/documentation/admin_api#get_product_environment_usage_details>`_
 
     :param options:     Additional options
     :type options:      dict, optional
@@ -54,42 +65,113 @@ def usage(**options):
     return call_api("get", uri, {}, **options)
 
 
+def config(**options):
+    """
+    Get account config details.
+
+    Fetches the account's configuration details with optional settings.
+
+    :param options: The optional parameters for the API request.
+    :type options:      dict, optional
+    :return:            Detailed config information.
+    :rtype:             Response
+    """
+    params = only(options, "settings")
+    return call_api("get", ["config"], params, **options)
+
+
 def resource_types(**options):
+    """
+    Retrieves the types of resources (assets) available.
+
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+    """
     return call_api("get", ["resources"], {}, **options)
 
 
 def resources(**options):
+    """
+    Retrieves resources (assets) based on the provided options.
+
+    :param options: Additional options to filter the resources.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+    """
     resource_type = options.pop("resource_type", "image")
     upload_type = options.pop("type", None)
     uri = ["resources", resource_type]
     if upload_type:
         uri.append(upload_type)
-    params = only(options, "next_cursor", "max_results", "prefix", "tags",
-                  "context", "moderations", "direction", "start_at", "metadata")
+    params = __list_resources_params(**options)
+    params.update(only(options, "prefix", "start_at"))
     return call_api("get", uri, params, **options)
 
 
 def resources_by_tag(tag, **options):
+    """
+    Lists resources (assets) with the specified tag.
+
+    This method does not return matching deleted assets, even if they have been backed up.
+
+    :param tag: The tag value.
+    :type tag: str
+    :param options: The optional parameters.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+
+    See: https://cloudinary.com/documentation/admin_api#get_resources_by_tag
+    """
     resource_type = options.pop("resource_type", "image")
     uri = ["resources", resource_type, "tags", tag]
-    params = only(options, "next_cursor", "max_results", "tags",
-                  "context", "moderations", "direction", "metadata")
+    params = __list_resources_params(**options)
     return call_api("get", uri, params, **options)
 
 
 def resources_by_moderation(kind, status, **options):
+    """
+    Lists resources (assets) assets currently in the specified moderation queue and status.
+
+    :param kind: Type of image moderation queue to list.
+                 Valid values: "manual", "webpurify", "aws_rek", or "metascan".
+    :type kind: str
+    :param status: Only assets with this moderation status will be returned.
+                   Valid values: "pending", "approved", "rejected".
+    :type status: str
+    :param options: The optional parameters.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+
+    See: https://cloudinary.com/documentation/admin_api#get_resources_in_moderation_queues
+    """
     resource_type = options.pop("resource_type", "image")
     uri = ["resources", resource_type, "moderations", kind, status]
-    params = only(options, "next_cursor", "max_results", "tags",
-                  "context", "moderations", "direction", "metadata")
+    params = __list_resources_params(**options)
     return call_api("get", uri, params, **options)
 
 
 def resources_by_ids(public_ids, **options):
+    """
+    Lists resources (assets) with the specified public IDs.
+
+    :param public_ids: The requested public_ids (up to 100).
+    :type public_ids: list[str]
+    :param options: The optional parameters.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+
+    See: https://cloudinary.com/documentation/admin_api#get_resources
+    """
     resource_type = options.pop("resource_type", "image")
     upload_type = options.pop("type", "upload")
     uri = ["resources", resource_type, upload_type]
-    params = dict(only(options, "tags", "moderations", "context"), public_ids=public_ids)
+    params = dict(__resources_params(**options), public_ids=public_ids)
     return call_api("get", uri, params, **options)
 
 
@@ -105,7 +187,7 @@ def resources_by_asset_folder(asset_folder, **options):
     :rtype:             Response
     """
     uri = ["resources", "by_asset_folder"]
-    params = only(options, "max_results", "tags", "moderations", "context", "next_cursor")
+    params = __list_resources_params(**options)
     params["asset_folder"] = asset_folder
     return call_api("get", uri, params, **options)
 
@@ -115,7 +197,7 @@ def resources_by_asset_ids(asset_ids, **options):
     This method does not return deleted assets even if they have been backed up.
 
     See: `Get resources by context API reference
-    <https://cloudinary.com/documentation/admin_api#get_resources>`_
+    <https://cloudinary.com/documentation/admin_api#get_resources_by_asset_ids>`_
 
     :param asset_ids:   The requested asset IDs.
     :type asset_ids:    list[str]
@@ -125,7 +207,7 @@ def resources_by_asset_ids(asset_ids, **options):
     :rtype:             Response
     """
     uri = ["resources", 'by_asset_ids']
-    params = dict(only(options, "tags", "moderations", "context"), asset_ids=asset_ids)
+    params = dict(__resources_params(**options), asset_ids=asset_ids)
     return call_api("get", uri, params, **options)
 
 
@@ -147,15 +229,79 @@ def resources_by_context(key, value=None, **options):
     """
     resource_type = options.pop("resource_type", "image")
     uri = ["resources", resource_type, "context"]
-    params = only(options, "next_cursor", "max_results", "tags",
-                  "context", "moderations", "direction", "metadata")
+    params = __list_resources_params(**options)
     params["key"] = key
     if value is not None:
         params["value"] = value
     return call_api("get", uri, params, **options)
 
 
+def __resources_params(**options):
+    """
+       Prepares optional parameters for resources_* API calls.
+
+       :param options: Additional options
+       :return: Optional parameters
+
+       :internal
+       """
+    params = only(options, "tags", "context", "metadata", "moderations")
+    params["fields"] = options.get("fields") and utils.encode_list(utils.build_array(options["fields"]))
+    return params
+
+
+def __list_resources_params(**options):
+    """
+       Prepares optional parameters for resources_* API calls.
+
+       :param options: Additional options
+       :return: Optional parameters
+
+       :internal
+       """
+    resources_params = __resources_params(**options)
+    resources_params.update(only(options, "next_cursor", "max_results", "direction"))
+
+    return resources_params
+
+
+def visual_search(image_url=None, image_asset_id=None, text=None, image_file=None, **options):
+    """
+    Find images based on their visual content.
+
+    :param image_url:       The URL of an image.
+    :type image_url:        str
+    :param image_asset_id:  The asset_id of an image in your account.
+    :type image_asset_id:   str
+    :param text:            A textual description, e.g., "cat"
+    :type text:             str
+    :param image_file:      The image file.
+    :type image_file:       str|callable|Path|bytes
+    :param options:         Additional options
+    :type options:          dict, optional
+    :return:                Resources (assets) that were found
+    :rtype:                 Response
+    """
+    uri = ["resources", "visual_search"]
+    params = {"image_url": image_url, "image_asset_id": image_asset_id, "text": text,
+              "image_file": utils.handle_file_parameter(image_file, "file")}
+    return call_api("post", uri, params, **options)
+
+
 def resource(public_id, **options):
+    """
+    Returns the details of the specified asset and all its derived assets.
+
+    Note that if you only need details about the original asset, you can also use the `uploader.upload`
+    or `uploader.explicit` methods, which return the same information and are not rate limited.
+
+    :param public_id: The public ID of the resource.
+    :type public_id: str
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+    """
     resource_type = options.pop("resource_type", "image")
     upload_type = options.pop("type", "upload")
     uri = ["resources", resource_type, upload_type, public_id]
@@ -194,6 +340,16 @@ def _prepare_asset_details_params(**options):
 
 
 def update(public_id, **options):
+    """
+    Updates the details of a specified resource.
+
+    :param public_id: The public ID of the resource to update.
+    :type public_id: str
+    :param options: Additional options for the update operation.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+    """
     resource_type = options.pop("resource_type", "image")
     upload_type = options.pop("type", "upload")
     uri = ["resources", resource_type, upload_type, public_id]
@@ -204,11 +360,11 @@ def update(public_id, **options):
     if "tags" in options:
         params["tags"] = ",".join(utils.build_array(options["tags"]))
     if "face_coordinates" in options:
-        params["face_coordinates"] = utils.encode_double_array(
-            options.get("face_coordinates"))
+        params["face_coordinates"] = utils.encode_double_array(options.get("face_coordinates"))
     if "custom_coordinates" in options:
-        params["custom_coordinates"] = utils.encode_double_array(
-            options.get("custom_coordinates"))
+        params["custom_coordinates"] = utils.encode_double_array(options.get("custom_coordinates"))
+    if "regions" in options:
+        params["regions"] = utils.json_encode(options.get("regions"))
     if "context" in options:
         params["context"] = utils.encode_context(options.get("context"))
     if "metadata" in options:
@@ -237,6 +393,22 @@ def delete_resources(public_ids, **options):
     return call_api("delete", uri, params, **options)
 
 
+def delete_resources_by_asset_ids(asset_ids, **options):
+    """
+    Deletes resources (assets) by asset IDs.
+
+    :param asset_ids: The asset IDs of the assets to delete.
+    :type asset_ids: list[str]
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the command.
+    :rtype: dict
+    """
+    uri = ["resources"]
+    params = __delete_resource_params(options, asset_ids=asset_ids)
+    return call_json_api("delete", uri, params, **options)
+
+
 def delete_resources_by_prefix(prefix, **options):
     resource_type = options.pop("resource_type", "image")
     upload_type = options.pop("type", "upload")
@@ -295,6 +467,24 @@ def delete_derived_by_transformation(public_ids, transformations,
     return call_api("delete", uri, params, **options)
 
 
+def delete_backed_up_assets(asset_id, version_ids, **options):
+    """
+    Deletes backed up versions of a resource by asset IDs.
+
+    :param asset_id: The asset ID of the asset to update.
+    :type asset_id: str
+    :param version_ids: The array of version IDs.
+    :type version_ids: list[str]
+    :param options: Additional options.
+    :type options: dict, optional
+    :return: The result of the command.
+    :rtype: dict
+    """
+    uri = ["resources", "backup", asset_id]
+    params = {"version_ids": utils.build_array(version_ids)}
+    return call_json_api("delete", uri, params, **options)
+
+
 def add_related_assets(public_id, assets_to_relate, resource_type="image", type="upload", **options):
     """
     Relates an asset to other assets by public IDs.
@@ -376,6 +566,16 @@ def delete_related_assets_by_asset_ids(asset_id, assets_to_unrelate, **options):
 
 
 def tags(**options):
+    """
+    Lists all the tags currently used for a specified asset type.
+
+    :param options: The optional parameters.
+    :type options: dict, optional
+    :return: The result of the API call.
+    :rtype: Response
+
+    See: https://cloudinary.com/documentation/admin_api#get_tags
+    """
     resource_type = options.pop("resource_type", "image")
     uri = ["tags", resource_type]
     return call_api("get", uri, only(options, "next_cursor", "max_results", "prefix"), **options)
@@ -480,10 +680,6 @@ def create_upload_preset(**options):
     return call_api("post", uri, params, **options)
 
 
-def create_folder(path, **options):
-    return call_api("post", ["folders", path], {}, **options)
-
-
 def root_folders(**options):
     return call_api("get", ["folders"], only(options, "next_cursor", "max_results"), **options)
 
@@ -492,6 +688,24 @@ def subfolders(of_folder_path, **options):
     return call_api("get", ["folders", of_folder_path], only(options, "next_cursor", "max_results"), **options)
 
 
+def create_folder(path, **options):
+    return call_api("post", ["folders", path], {}, **options)
+
+
+def rename_folder(from_path, to_path, **options):
+    """
+    Renames folder
+
+    :param from_path: The full path of an existing asset folder.
+    :param to_path:   The full path of the new asset folder.
+    :param options:   Additional options
+
+    :rtype: Response
+    """
+    params = {"to_folder": to_path}
+    return call_api("put", ["folders", from_path], params, **options)
+
+
 def delete_folder(path, **options):
     """Deletes folder
 
@@ -502,7 +716,9 @@ def delete_folder(path, **options):
 
     :rtype: Response
     """
-    return call_api("delete", ["folders", path], {}, **options)
+
+    params = only(options, "skip_backup")
+    return call_api("delete", ["folders", path], params, **options)
 
 
 def restore(public_ids, **options):
@@ -513,6 +729,22 @@ def restore(public_ids, **options):
     return call_json_api("post", uri, params, **options)
 
 
+def restore_by_asset_ids(asset_ids, **options):
+    """
+      Restores resources (assets) by their asset IDs.
+
+      :param asset_ids: The asset IDs of the assets to restore.
+      :type asset_ids: list[str]
+      :param options: Additional options.
+      :type options: dict, optional
+      :return: The result of the restore operation.
+      :rtype: dict
+      """
+    uri = ["resources", "restore"]
+    params = dict(asset_ids=asset_ids, **only(options, "versions"))
+    return call_json_api("post", uri, params, **options)
+
+
 def upload_mappings(**options):
     uri = ["upload_mappings"]
     return call_api("get", uri, only(options, "next_cursor", "max_results"), **options)
@@ -636,9 +868,8 @@ def add_metadata_field(field, **options):
 
     :rtype: Response
     """
-    params = only(field, "type", "external_id", "label", "mandatory",
-                  "default_value", "validation", "datasource")
-    return call_metadata_api("post", [], params, **options)
+
+    return call_metadata_api("post", [], __metadata_field_params(field), **options)
 
 
 def update_metadata_field(field_external_id, field, **options):
@@ -657,8 +888,13 @@ def update_metadata_field(field_external_id, field, **options):
     :rtype: Response
     """
     uri = [field_external_id]
-    params = only(field, "label", "mandatory", "default_value", "validation")
-    return call_metadata_api("put", uri, params, **options)
+
+    return call_metadata_api("put", uri, __metadata_field_params(field), **options)
+
+
+def __metadata_field_params(field):
+    return only(field, "type", "external_id", "label", "mandatory", "restrictions",
+                "default_value", "default_disabled", "validation", "datasource", "allow_dynamic_list_values")
 
 
 def delete_metadata_field(field_external_id, **options):
@@ -778,3 +1014,18 @@ def reorder_metadata_fields(order_by, direction=None, **options):
     uri = ['order']
     params = {'order_by': order_by, 'direction': direction}
     return call_metadata_api('put', uri, params, **options)
+
+
+def analyze(input_type, analysis_type, uri=None, **options):
+    """Analyzes an asset with the requested analysis type.
+
+    :param input_type: The type of input for the asset to analyze ('uri').
+    :param analysis_type: The type of analysis to run ('google_tagging', 'captioning', 'fashion').
+    :param uri: The URI of the asset to analyze.
+    :param options: Additional options.
+
+    :rtype: Response
+    """
+    api_uri = ['analysis', 'analyze', input_type]
+    params = {'analysis_type': analysis_type, 'uri': uri, 'parameters': options.get("parameters")}
+    return _call_v2_api('post', api_uri, params, **options)
