@@ -8,7 +8,7 @@ import cloudinary.provisioning.account
 from cloudinary.provisioning import account_config, reset_config
 from cloudinary.exceptions import AuthorizationRequired, NotFound
 
-from test.helper_test import UNIQUE_SUB_ACCOUNT_ID
+from test.helper_test import UNIQUE_SUB_ACCOUNT_ID, UNIQUE_TEST_ID
 
 disable_warnings()
 
@@ -161,6 +161,17 @@ class AccountApiTest(unittest.TestCase):
         with six.assertRaisesRegex(self, NotFound, "Cannot find sub account with id {}".format(UNIQUE_SUB_ACCOUNT_ID)):
             cloudinary.provisioning.users(pending=True, sub_account_id=UNIQUE_SUB_ACCOUNT_ID)
 
+    @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
+                         "requires provisioning_api_key/provisioning_api_secret")
+    def test_get_users_by_login(self):
+        res = cloudinary.provisioning.users(user_ids=[self.user_id_1], pending=None, 
+                                            last_login="true", from_date=datetime.today(), to_date=datetime.today())
+        self.assertEqual(len(res["users"]), 0)
+
+        res = cloudinary.provisioning.users(user_ids=[self.user_id_1], pending=None,
+                                            last_login="false", from_date=datetime.today(), to_date=datetime.today())
+        self.assertEqual(len(res["users"]), 1)
+
     @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
                          "requires provisioning_api_key/provisioning_api_secret")
     def test_update_user_group(self):
@@ -196,6 +207,62 @@ class AccountApiTest(unittest.TestCase):
         # Ensure we can find our ID in the list(Which means we got a real list as a response)
         self.assertEqual(group_by_id[0]["id"], self.group_id)
 
+    @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
+                         "requires provisioning_api_key/provisioning_api_secret")
+    def test_get_access_keys(self):
+        res = cloudinary.provisioning.access_keys(self.cloud_id)
+
+        self.assertGreater(res["total"], 0)
+        self.assertGreater(len(res["access_keys"]), 0)
+
+    @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
+                         "requires provisioning_api_key/provisioning_api_secret")
+    def test_generate_access_key(self):
+        key_name = UNIQUE_TEST_ID + "_test_key"
+        res = cloudinary.provisioning.generate_access_key(self.cloud_id, name=key_name, enabled=False)
+
+        self.assertEqual(key_name, res["name"])
+        self.assertEqual(False, res["enabled"])
+
+    @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
+                         "requires provisioning_api_key/provisioning_api_secret")
+    def test_update_access_key(self):
+        key_name = UNIQUE_TEST_ID + "_before_update_test_key"
+        updated_key_name = UNIQUE_TEST_ID + "_updated_test_key"
+
+        key_res = cloudinary.provisioning.generate_access_key(self.cloud_id, name=key_name, enabled=False)
+
+        self.assertEqual(key_name, key_res["name"])
+        self.assertEqual(False, key_res["enabled"])
+
+        res = cloudinary.provisioning.update_access_key(self.cloud_id, key_res["api_key"],
+                                                        name=updated_key_name, enabled=True, dedicated_for="webhooks")
+
+        self.assertEqual(updated_key_name, res["name"])
+        self.assertEqual(True, res["enabled"])
+        self.assertEqual(1, len(res["dedicated_for"]))
+        self.assertEqual("webhooks", res["dedicated_for"][0])
+
+    @unittest.skipUnless(cloudinary.provisioning.account_config().provisioning_api_secret,
+                         "requires provisioning_api_key/provisioning_api_secret")
+    def test_delete_access_key(self):
+        key_name = UNIQUE_TEST_ID + "_delete_key"
+        named_key_name = UNIQUE_TEST_ID + "_delete_by_name_key"
+
+        key_res = cloudinary.provisioning.generate_access_key(self.cloud_id, name=key_name, enabled=True)
+        self.assertEqual(key_name, key_res["name"])
+        self.assertEqual(True, key_res["enabled"])
+
+        named_key_res = cloudinary.provisioning.generate_access_key(self.cloud_id, name=named_key_name, enabled=True)
+        self.assertEqual(named_key_name, named_key_res["name"])
+        self.assertEqual(True, named_key_res["enabled"])
+
+        key_del_res = cloudinary.provisioning.delete_access_key(self.cloud_id, named_key_res["api_key"])
+        self.assertEqual("ok", key_del_res["message"])
+
+        named_key_del_res = cloudinary.provisioning.delete_access_key(self.cloud_id, name=key_name)
+        self.assertEqual("ok", named_key_del_res["message"])
+
 
 if __name__ == '__main__':
     unittest.main()
