@@ -4,7 +4,6 @@ import re
 import tempfile
 import unittest
 import uuid
-import json
 import time
 from collections import OrderedDict
 from datetime import datetime, date
@@ -12,7 +11,6 @@ from fractions import Fraction
 from os.path import getsize
 
 import six
-from mock import patch
 
 import cloudinary.utils
 from cloudinary import CL_BLANK
@@ -32,7 +30,7 @@ from cloudinary.utils import (
     verify_api_response_signature,
 )
 from cloudinary.compat import to_bytes
-from test.helper_test import TEST_IMAGE, REMOTE_TEST_IMAGE
+from test.helper_test import TEST_IMAGE, REMOTE_TEST_IMAGE, patch
 from test.test_api import (
     API_TEST_TRANS_SCALE100,
     API_TEST_TRANS_SCALE100_STR,
@@ -47,7 +45,8 @@ DEFAULT_FETCH_PATH = DEFAULT_ROOT_PATH + 'image/fetch/'
 VIDEO_UPLOAD_PATH = DEFAULT_ROOT_PATH + 'video/upload/'
 TEST_ID = 'test'
 
-FETCH_URL = "http://cloudinary.com/images/logo.png"
+FETCH_URL = "https://cloudinary.com/images/logo.png"
+FETCH_VIDEO_URL = "https://demo-res.cloudinary.com/videos/dog.mp4"
 
 IMAGE_VERSION = "1234"
 IMAGE_VERSION_STR = "v" + IMAGE_VERSION
@@ -57,6 +56,7 @@ TEST_FOLDER = 'folder/test'
 MOCKED_NOW = 1549533574
 API_SECRET = 'X7qLTrsES31MzxxkxPPA-pAGGfU'
 
+
 class TestUtils(unittest.TestCase):
     crop_transformation = {'crop': 'crop', 'width': 100}
     crop_transformation_str = 'c_crop,w_100'
@@ -438,6 +438,14 @@ class TestUtils(unittest.TestCase):
                     + "l_fetch:aHR0cDovL2Nsb3VkaW5hcnkuY29tL2ltYWdlcy9vbGRfbG9nby5wbmc=/"
                     + "test"))
 
+        """should support video overlay"""
+        self.__test_cloudinary_url(
+            options={"overlay": "video:fetch:" + FETCH_VIDEO_URL},
+            expected_url=(
+                    DEFAULT_UPLOAD_PATH
+                    + "l_video:fetch:aHR0cHM6Ly9kZW1vLXJlcy5jbG91ZGluYXJ5LmNvbS92aWRlb3MvZG9nLm1wNA==/"
+                    + "test"))
+
         self.__test_cloudinary_url(
             options={
                 "overlay": {
@@ -446,8 +454,18 @@ class TestUtils(unittest.TestCase):
             expected_url=(
                     DEFAULT_UPLOAD_PATH +
                     "l_fetch:"
-                    "aHR0cHM6Ly91cGxvYWQud2lraW1lZGlhLm9yZy93aWtpcGVkaWEvY29"
-                    "tbW9ucy8yLzJiLyVFQSVCMyVBMCVFQyVCMCVCRCVFQSVCMCVBRiVFQiVCMiU4Qy5qcGc=/"
+                    "aHR0cHM6Ly91cGxvYWQud2lraW1lZGlhLm9yZy93aWtpcGVkaWEvY29tbW9ucy8yLzJiL-qzoOywveqwr-uyjC5qcGc=/"
+                    "test"))
+
+        self.__test_cloudinary_url(
+            options={
+                "overlay": {
+                    "url":
+                        "https://www.test.com/test/JE01118-YGP900_1_lar.jpg?version=432023"}},
+            expected_url=(
+                    DEFAULT_UPLOAD_PATH +
+                    "l_fetch:"
+                    "aHR0cHM6Ly93d3cudGVzdC5jb20vdGVzdC9KRTAxMTE4LVlHUDkwMF8xX2xhci5qcGc_dmVyc2lvbj00MzIwMjM=/"
                     "test"))
 
     def test_underlay(self):
@@ -640,6 +658,18 @@ class TestUtils(unittest.TestCase):
             options={"version": 1234, "type": "fetch", "sign_url": True},
             expected_url=DEFAULT_ROOT_PATH + "image/fetch/s--hH_YcbiS--/v1234/http://google.com/path/to/image.png")
 
+        self.__test_cloudinary_url(
+            public_id="image.jpg",
+            options={"type": "authenticated", "transformation": {"color": "red", "overlay": {"text": "Cool%F0%9F%98%8D", "font_family": "Times", "font_size": 70, "font_weight": "bold"}},
+                     "sign_url": True},
+            expected_url="http://res.cloudinary.com/test123/image/authenticated/s--Uqk1a-5W--/co_red,l_text:Times_70_bold:Cool%25F0%259F%2598%258D/image.jpg")
+
+        self.__test_cloudinary_url(
+            public_id="image.jpg",
+            options={"type": "authenticated", "transformation": {"raw_transformation": "co_red,l_text:Times_70_bold:Cool%25F0%259F%2598%258D"},
+                     "sign_url": True},
+            expected_url="http://res.cloudinary.com/test123/image/authenticated/s--Uqk1a-5W--/co_red,l_text:Times_70_bold:Cool%25F0%259F%2598%258D/image.jpg")
+
     def test_signed_url_sha256(self):
         sha256_config = cloudinary.Config()
         sha256_config.update(**vars(cloudinary.config()))
@@ -939,8 +969,9 @@ class TestUtils(unittest.TestCase):
               'font_family': "Arial", 'font_size': 40},
              "subtitles:Arial_40:sample_sub_he.srt"),
             ({'url': "https://upload.wikimedia.org/wikipedia/commons/2/2b/고창갯벌.jpg"},
-             "fetch:aHR0cHM6Ly91cGxvYWQud2lraW1lZGlhLm9yZy93aWtpcGVkaWEvY29"
-             "tbW9ucy8yLzJiLyVFQSVCMyVBMCVFQyVCMCVCRCVFQSVCMCVBRiVFQiVCMiU4Qy5qcGc=")
+             "fetch:aHR0cHM6Ly91cGxvYWQud2lraW1lZGlhLm9yZy93aWtpcGVkaWEvY29tbW9ucy8yLzJiL-qzoOywveqwr-uyjC5qcGc="),
+            ({'url': FETCH_VIDEO_URL, "resource_type": "video"},
+             "video:fetch:aHR0cHM6Ly9kZW1vLXJlcy5jbG91ZGluYXJ5LmNvbS92aWRlb3MvZG9nLm1wNA==")
         ]
 
         for options, expected in tests:
